<?php
function build_map_html_data()
{
    require_once __DIR__ . '/RoborockMapParser.php';
    $out = [];
// Проверка наличия файла карты
    $gzipFile = __DIR__ . '/map.gz';
    if (!file_exists($gzipFile)) {
        $out['ERROR'] = "Ошибка: файл map.gz не найден. Попробуйте обновить карту.";
    } else {
        $rooms = null;
        try {
            // Загрузка и парсинг данных карты
            $gzipData = file_get_contents($gzipFile);
            $parser = new RoborockMapParser($gzipData);
            $parser->parse();
            $rooms = $parser->getRooms();      // Формат: [roomId => ['bbox'=>[xmin,ymin,xmax,ymax], 'color'=>[r,g,b]]]
            $width = $parser->getWidth();
            $height = $parser->getHeight();
        } catch (\Throwable $e) {
            $out['ERROR'] = "Ошибка при парсинге map.gz: " . htmlspecialchars($e->getMessage());
        }

        if ($rooms != null) {
            $out['WIDTH'] = $width;
            $out['HEIGHT'] = $height;
            $roomMapping = roborock_getRoomMapping();

            // Обработка каждой комнаты на карте
            foreach ($rooms as $roomId => $r) {
                [$xmin, $ymin, $xmax, $ymax] = $r['bbox'];
                $w = $xmax - $xmin;
                $h = $ymax - $ymin;

                // Пропуск комнат с нулевыми размерами
                if ($w <= 0 || $h <= 0) {
                    continue;
                }

                // Определение индекса комнаты
                $roomIndex = -1;
                for ($i = 0; $i < sizeof($roomMapping); $i++) {
                    if ($roomMapping[$i][0] == $roomId) {
                        $roomIndex = $i + 1;
                        break;
                    }
                }

                // Генерация HTML-разметки для комнаты
                $out['DIV'][]['HTML'] = '<div class="room-area" '
                    . 'data-roomid="' . intval($roomId) . '" '
                    . 'data-xmin="' . intval($xmin) . '" '
                    . 'data-ymin="' . intval($ymin) . '" '
                    . 'data-xmax="' . intval($xmax) . '" '
                    . 'data-ymax="' . intval($ymax) . '" '
                    . 'style="left:' . intval($xmin) . 'px; top:' . intval($ymin) . 'px; '
                    . 'width:' . intval($w) . 'px; height:' . intval($h) . 'px;">'
                    . '<div class="room-label">'
                    . '<span class="checkmark">✓</span>'
                    . '<span class="room-index">' . $roomIndex . '</span>'
                    . '</div>'
                    . '</div>';
            }
        }
    }
    return $out;
}