<?php
/**
 * Модуль управления пылесосом Roborock
 * @package project
 * @author Wizard <sergejey@gmail.com>
 * @copyright http://majordomo.smartliving.ru/ (c)
 * @version 0.1 (wizard, 23:05:38 [May 31, 2025])
 */
//
//
class roborockvacuum extends module
{
    /**
     * Конструктор класса модуля
     * @access private
     */
    function __construct()
    {
        $this->name = "roborockvacuum";
        $this->title = "Roborock";
        $this->module_category = "<#LANG_SECTION_DEVICES#>";
        $this->checkInstalled();
    }

    /**
     * Сохранение параметров модуля
     * @access public
     */
    function saveParams($data = 1)
    {
        $p = array();
        if (isset($this->id)) {
            $p["id"] = $this->id;
        }
        if (isset($this->view_mode)) {
            $p["view_mode"] = $this->view_mode;
        }
        if (isset($this->edit_mode)) {
            $p["edit_mode"] = $this->edit_mode;
        }
        if (isset($this->tab)) {
            $p["tab"] = $this->tab;
        }
        return parent::saveParams($p);
    }

    /**
     * Получение параметров модуля из строки запроса
     * @access public
     */
    function getParams()
    {
        global $id;
        global $mode;
        global $view_mode;
        global $edit_mode;
        global $tab;
        if (isset($id)) {
            $this->id = $id;
        }
        if (isset($mode)) {
            $this->mode = $mode;
        }
        if (isset($view_mode)) {
            $this->view_mode = $view_mode;
        }
        if (isset($edit_mode)) {
            $this->edit_mode = $edit_mode;
        }
        if (isset($tab)) {
            $this->tab = $tab;
        }
    }

    /**
     * Основной метод выполнения модуля
     * @access public
     */
    function run()
    {
        global $session;
        $out = array();

        // Проверяем, вызван ли модуль через шаблон
        if (isset($this->template_mode) && $this->template_mode) {
            $this->getConfig();
            require_once 'functions.php';
            $this->data = build_map_html_data();
            $p = new parser(DIR_TEMPLATES . $this->name . "/" . $this->name . "_search_admin.html", $this->data, $this);
            $this->result = $p->result;
            return;
        }

        if ($this->action == 'admin') {
            $this->admin($out);
        } else {
            $this->usual($out);
        }
        if (isset($this->owner->action)) {
            $out['PARENT_ACTION'] = $this->owner->action;
        }
        if (isset($this->owner->name)) {
            $out['PARENT_NAME'] = $this->owner->name;
        }
        $out['VIEW_MODE'] = $this->view_mode;
        $out['EDIT_MODE'] = $this->edit_mode;
        $out['MODE'] = $this->mode;
        $out['ACTION'] = $this->action;
        $out['TAB'] = $this->tab;
        $this->data = $out;
        $p = new parser(DIR_TEMPLATES . $this->name . "/" . $this->name . ".html", $this->data, $this);
        $this->result = $p->result;
    }

    /**
     * Административный интерфейс модуля
     * @access public
     */
    function admin(&$out)
    {
        $this->getConfig();
        $out['API_URL'] = $this->config['API_URL'];
        if (!$out['API_URL']) {
            $out['API_URL'] = 'http://';
        }
        $out['API_TOKEN'] = $this->config['API_TOKEN'];
        $out['MI_USERNAME'] = $this->config['MI_USERNAME'];
        $out['MI_PASSWORD'] = $this->config['MI_PASSWORD'];
        $out['VACUUM_IP'] = $this->config['VACUUM_IP'];
        $out['REGION_CODE'] = $this->config['REGION_CODE'];
        if (!$out['REGION_CODE']) {
            $out['REGION_CODE'] = 'ru';
        }
        if ($this->view_mode == 'update_settings') {
            global $api_url;
            $this->config['API_URL'] = $api_url;
            global $api_token;
            $this->config['API_TOKEN'] = $api_token;
            global $mi_username;
            $this->config['MI_USERNAME'] = $mi_username;
            global $mi_password;
            $this->config['MI_PASSWORD'] = $mi_password;
            global $vacuum_ip;
            $this->config['VACUUM_IP'] = $vacuum_ip;
            global $region_code;
            $this->config['REGION_CODE'] = $region_code;
            $this->saveConfig();
            $this->redirect("?");
        }
        if (isset($this->data_source) && !$_GET['data_source'] && !$_POST['data_source']) {
            $out['SET_DATASOURCE'] = 1;
        }
        if ($this->data_source == 'roborockvacuum' || $this->data_source == '') {
            if ($this->view_mode == '' || $this->view_mode == 'search_roborockvacuum') {
                if ($_GET['mode2'] == 'download') {
                    $this->download_roborockvacuum($out, $this->id);
                }
                $this->search_roborockvacuum($out);
            }
            if ($this->view_mode == 'edit_roborockvacuum') {
                $this->edit_roborockvacuum($out, $this->id);
            }
            if ($this->view_mode == 'delete_roborockvacuum') {
                $this->delete_roborockvacuum($this->id);
                $this->redirect("?");
            }
        }
    }

    /**
     * Пользовательский интерфейс модуля
     * @access public
     */
    function usual(&$out)
    {
        $this->admin($out);
    }

    /**
     * Поиск устройств Roborock
     * @access public
     */
    function search_roborockvacuum(&$out)
    {
        require(dirname(__FILE__) . '/roborockvacuum_search.inc.php');
    }

    /**
     * Редактирование/добавление устройства Roborock
     * @access public
     */
    function edit_roborockvacuum(&$out, $id)
    {
        require(dirname(__FILE__) . '/roborockvacuum_edit.inc.php');
    }

    /**
     * Загрузка данных устройства Roborock
     * @access public
     */
    function download_roborockvacuum(&$out, $id)
    {
        require(dirname(__FILE__) . '/downloader.php');
//        exit;
    }

    /**
     * Удаление устройства Roborock
     * @access public
     */
    function delete_roborockvacuum($id)
    {
        $rec = SQLSelectOne("SELECT * FROM roborockvacuum WHERE ID='$id'");
        // some action for related tables
        SQLExec("DELETE FROM roborockvacuum WHERE ID='" . $rec['ID'] . "'");
    }

    /**
     * Обработчик изменения свойств объекта
     * @access public
     */
    function propertySetHandle($object, $property, $value)
    {
        $this->getConfig();
        $table = 'roborockvacuum';
        $properties = SQLSelect("SELECT ID FROM $table WHERE LINKED_OBJECT LIKE '" . DBSafe($object) . "' AND LINKED_PROPERTY LIKE '" . DBSafe($property) . "'");
        $total = count($properties);
        if ($total) {
            for ($i = 0; $i < $total; $i++) {
                //to-do
            }
        }
    }

    /**
     * Циклическая обработка состояния устройства
     * @access public
     */
    function processCycle()
    {
        if (!function_exists('socket_create') ||
            !function_exists('openssl_encrypt') ||
            !function_exists('json_encode')) {
            return;
        }
        $this->getConfig();
        $status = roborock_getStatus();
        setGlobal("RoborockVacuum1.state", roborock_getStatusString($status['state']));
        SQLExec("UPDATE pvalues SET VALUE=" . $status['fan_power'] . " WHERE PROPERTY_NAME='RoborockVacuum1.fan_power'");
        SQLExec("UPDATE pvalues SET VALUE=" . $status['water_box_mode'] . " WHERE PROPERTY_NAME='RoborockVacuum1.water_flow'");
        SQLExec("UPDATE pvalues SET VALUE=" . $status['mop_mode'] . " WHERE PROPERTY_NAME='RoborockVacuum1.mop_mode'");
        setGlobal('RoborockVacuum1.battery', $status['battery']);

        // Update consumable information
        $consumable = roborock_getConsumable();
        if ($consumable) {
            setGlobal('RoborockVacuum1.main_brush_work_time', $consumable['main_brush_work_time']);
            setGlobal('RoborockVacuum1.side_brush_work_time', $consumable['side_brush_work_time']);
            setGlobal('RoborockVacuum1.filter_work_time', $consumable['filter_work_time']);
            setGlobal('RoborockVacuum1.filter_element_work_time', $consumable['filter_element_work_time']);
            setGlobal('RoborockVacuum1.sensor_dirty_time', $consumable['sensor_dirty_time']);
            setGlobal('RoborockVacuum1.strainer_work_times', $consumable['strainer_work_times']);
            setGlobal('RoborockVacuum1.dust_collection_work_times', $consumable['dust_collection_work_times']);
            setGlobal('RoborockVacuum1.cleaning_brush_work_times', $consumable['cleaning_brush_work_times']);

            $percent_main_brush = min(100, round(($consumable['main_brush_work_time'] / 3600) / 300 * 100));
            $percent_side_brush = min(100, round(($consumable['side_brush_work_time'] / 3600) / 200 * 100));
            $percent_filter = min(100, round(($consumable['filter_work_time'] / 3600) / 150 * 100));
            $percent_sensor_dirty = min(100, round(($consumable['sensor_dirty_time'] / 3600) / 30 * 100));
            setGlobal('RoborockVacuum1.main_brush_usage_percent', $percent_main_brush);
            setGlobal('RoborockVacuum1.side_brush_usage_percent', $percent_side_brush);
            setGlobal('RoborockVacuum1.filter_usage_percent', $percent_filter);
            setGlobal('RoborockVacuum1.sensor_dirty_usage_percent', $percent_sensor_dirty);
        }
    }

    function addClassPropertyWithMethod($property_name, $method_name, $fun_code)
    {
        addClassMethod('RoborockVacuum', $method_name, $fun_code);

        if ($prop_id = addClassProperty('RoborockVacuum', $property_name)) {
            $property = SQLSelectOne('SELECT * FROM `properties` WHERE `ID` = ' . $prop_id);
            $property['ONCHANGE'] = $method_name;
            SQLUpdate('properties', $property);
        }
    }

    /**
     * Добавление свойств и методов для комнат
     * @access public
     */
    function addRoomsPropertiesAndMethods($type)
    {
        for ($i = 1; $i <= 5; $i++) {
            $room_name = 'room' . $i;
            $property_name = $type . '_' . $room_name;
            $method_name = $type . ucfirst($room_name);
            if ($type == 'clean') {
                $fun_code = 'callMethod("RoborockVacuum1.cleanRoom", ["NEW_VALUE" => $params["NEW_VALUE"], "ROOM" => ' . $i . ']);
                setGlobal("RoborockVacuum1.' . $property_name . '", 0);';
            } elseif ($type == 'sweep') {
                $fun_code = 'callMethod("RoborockVacuum1.sweepRoom", ["NEW_VALUE" => $params["NEW_VALUE"], "ROOM" => ' . $i . ']);
                setGlobal("RoborockVacuum1.' . $property_name . '", 0);';
            } elseif ($type == 'mop') {
                $fun_code = 'callMethod("RoborockVacuum1.mopRoom", ["NEW_VALUE" => $params["NEW_VALUE"], "ROOM" => ' . $i . ']);
                setGlobal("RoborockVacuum1.' . $property_name . '", 0);';
            } else {
                $fun_code = false;
            }

            if ($fun_code) {
                $this->addClassPropertyWithMethod($property_name, $method_name, $fun_code);
            }
        }
    }

    /**
     * Install
     *
     * Module installation routine
     *
     * @access private
     */
    function install($data = '')
    {
        addClass('RoborockVacuum');
        addClassMethod('RoborockVacuum', 'cleanRoom', 'if (isset($params["ROOM"]) && (!isset($params["NEW_VALUE"]) || $params["NEW_VALUE"])) {
            $rooms = json_decode(getGlobal("RoborockVacuum1.rooms"));
            if (isset($rooms[$params["ROOM"] - 1]) && is_array($rooms[$params["ROOM"] - 1]) && sizeof($rooms[$params["ROOM"] - 1]) > 0) {
                $room_id = $rooms[$params["ROOM"] - 1][0];
                roborock_appCleanRooms([$room_id]);
            }
        }');
        addClassMethod('RoborockVacuum', 'sweepRoom', 'if (isset($params["ROOM"]) && (!isset($params["NEW_VALUE"]) || $params["NEW_VALUE"])) {
            $rooms = json_decode(getGlobal("RoborockVacuum1.rooms"));
            if (isset($rooms[$params["ROOM"] - 1]) && is_array($rooms[$params["ROOM"] - 1]) && sizeof($rooms[$params["ROOM"] - 1]) > 0) {
                $room_id = $rooms[$params["ROOM"] - 1][0];
                roborock_setCleanMotorMode(ROBOROCK_FAN_POWER_TURBO, ROBOROCK_MOP_MODE_STANDARD, ROBOROCK_WATER_FLOW_OFF);
                roborock_appCleanRooms([$room_id]);
            }
        }');
        addClassMethod('RoborockVacuum', 'mopRoom', 'if (isset($params["ROOM"]) && (!isset($params["NEW_VALUE"]) || $params["NEW_VALUE"])) {
            $rooms = json_decode(getGlobal("RoborockVacuum1.rooms"));
            if (isset($rooms[$params["ROOM"] - 1]) && is_array($rooms[$params["ROOM"] - 1]) && sizeof($rooms[$params["ROOM"] - 1]) > 0) {
                $room_id = $rooms[$params["ROOM"] - 1][0];
                roborock_setCleanMotorMode(ROBOROCK_FAN_POWER_OFF, ROBOROCK_MOP_MODE_DEEP_PLUS, ROBOROCK_WATER_FLOW_HIGH);
                roborock_appCleanRooms([$room_id]);
            }
        }');
        addClassProperty('RoborockVacuum', 'rooms');
        $this->addClassPropertyWithMethod('fan_power', 'onFanPowerChange', 'roborock_setCleanMotorMode($params["NEW_VALUE"], getGlobal("RoborockVacuum1.mop_mode"), getGlobal("RoborockVacuum1.water_flow"));');
        $this->addClassPropertyWithMethod('water_flow', 'onWaterFlowChange', 'roborock_setCleanMotorMode(getGlobal("RoborockVacuum1.fan_power"), getGlobal("RoborockVacuum1.mop_mode"), $params["NEW_VALUE"]);');
        $this->addClassPropertyWithMethod('mop_mode', 'onMopModeChange', 'roborock_setCleanMotorMode(getGlobal("RoborockVacuum1.fan_power"), $params["NEW_VALUE"], getGlobal("RoborockVacuum1.water_flow"));');
        addClassProperty('RoborockVacuum', 'battery');
        addClassProperty('RoborockVacuum', 'state');
        addClassProperty('RoborockVacuum', 'main_brush_work_time');
        addClassProperty('RoborockVacuum', 'side_brush_work_time');
        addClassProperty('RoborockVacuum', 'filter_work_time');
        addClassProperty('RoborockVacuum', 'filter_element_work_time');
        addClassProperty('RoborockVacuum', 'sensor_dirty_time');
        addClassProperty('RoborockVacuum', 'strainer_work_times');
        addClassProperty('RoborockVacuum', 'dust_collection_work_times');
        addClassProperty('RoborockVacuum', 'cleaning_brush_work_times');
        addClassProperty('RoborockVacuum', 'main_brush_usage_percent');
        addClassProperty('RoborockVacuum', 'side_brush_usage_percent');
        addClassProperty('RoborockVacuum', 'filter_usage_percent');
        addClassProperty('RoborockVacuum', 'sensor_dirty_usage_percent');

        $this->addClassPropertyWithMethod('clean_all', 'cleanAll', 'roborock_appStart();');
        $this->addClassPropertyWithMethod('sweep_all', 'sweepAll', 'roborock_setCleanMotorMode(ROBOROCK_FAN_POWER_TURBO, ROBOROCK_MOP_MODE_STANDARD, ROBOROCK_WATER_FLOW_OFF);
        roborock_appStart();');
        $this->addClassPropertyWithMethod('mop_all', 'mopAll', 'roborock_appStartWash();');
        $this->addRoomsPropertiesAndMethods('clean');
        $this->addRoomsPropertiesAndMethods('sweep');
        $this->addRoomsPropertiesAndMethods('mop');
        $this->addClassPropertyWithMethod('charge', 'charge', 'if ($params["NEW_VALUE"]) {
        roborock_appCharge();
        setGlobal("RoborockVacuum1.charge", 0);
        }');

        addClassObject('RoborockVacuum', 'RoborockVacuum1');
        parent::install();
    }

    /**
     * Uninstall
     *
     * Module uninstall routine
     *
     * @access public
     */
    function uninstall()
    {
        parent::uninstall();
    }

    /**
     * dbInstall
     *
     * Database installation routine
     *
     * @access private
     */
    function dbInstall($data)
    {
//        /*
//        roborockvacuum -
//        */
//        $data = <<<EOD
// roborockvacuum: ID int(10) unsigned NOT NULL auto_increment
// roborockvacuum: TITLE varchar(100) NOT NULL DEFAULT ''
// roborockvacuum: LINKED_OBJECT varchar(100) NOT NULL DEFAULT ''
// roborockvacuum: LINKED_PROPERTY varchar(100) NOT NULL DEFAULT ''
// roborockvacuum: LINKED_METHOD varchar(100) NOT NULL DEFAULT ''
// roborockvacuum: UPDATED datetime
//EOD;
//        parent::dbInstall($data);
    }
// --------------------------------------------------------------------
}
/*
*
* TW9kdWxlIGNyZWF0ZWQgTWF5IDMxLCAyMDI1IHVzaW5nIFNlcmdlIEouIHdpemFyZCAoQWN0aXZlVW5pdCBJbmMgd3d3LmFjdGl2ZXVuaXQuY29tKQ==
*
*/
