<?php
// modules/roborockvacuum/roborockvacuum_api.php

header('Content-Type: application/json; charset=UTF-8');

require_once __DIR__ . '/RoborockMapParser.php';

chdir(dirname(__FILE__) . '/../../');
include_once "./config.php";
include_once "./lib/loader.php";
include_once "./lib/threads.php";
set_time_limit(0);

// подключаем нашу библиотеку с функциями
require_once __DIR__ . '/../../lib/roborockvacuum.php';

$response = [
    'success' => false,
    'action'  => null,
    'data'    => null,
    'error'   => null,
    'message' => null,
];

$action = trim($_REQUEST['vacuum_action'] ?? '');
$response['action'] = $action;

try {
    if ($action === '') {
        throw new \Exception('Параметр vacuum_action не передан.');
    }

    switch ($action) {
        // Статус
        case 'get_status':
            $data = roborock_getStatus();
            break;

        // Звук
        case 'get_sound_volume':
            $data = roborock_getSoundVolume();
            break;
        case 'change_sound_volume':
            if (!isset($_REQUEST['volume'])) {
                throw new \Exception('Не передан параметр volume.');
            }
            $data = roborock_changeSoundVolume((int)$_REQUEST['volume']);
            break;
        case 'test_sound_volume':
            $data = roborock_testSoundVolume();
            break;

        // Не беспокоить
        case 'set_dnd_timer':
            if (!isset($_REQUEST['timer'])) {
                throw new \Exception('Не передан параметр timer.');
            }
            $arr = json_decode($_REQUEST['timer'], true);
            if (!is_array($arr) || count($arr) !== 4) {
                throw new \Exception('Неверный формат timer. Ожидается JSON-массив из четырёх чисел.');
            }
            list($sh, $sm, $eh, $em) = $arr;
            $data = roborock_setDndTimer($sh, $sm, $eh, $em);
            break;
        case 'close_dnd_timer':
            $data = roborock_closeDndTimer();
            break;

        // Мотор, мощность и вода
        case 'get_clean_motor_mode':
            $data = roborock_getCleanMotorMode();
            break;
        case 'set_clean_motor_mode':
            if (!isset($_REQUEST['param'])) {
                throw new \Exception('Не передан параметр param.');
            }
            $arrRaw = json_decode($_REQUEST['param'], true);
            if (!is_array($arrRaw) || count($arrRaw) === 0) {
                throw new \Exception('Неверный формат param для set_clean_motor_mode.');
            }
            $cfg = $arrRaw[0];
            foreach (['fan_power','mop_mode','water_box_mode'] as $key) {
                if (!isset($cfg[$key])) {
                    throw new \Exception("Не передан параметр {$key}.");
                }
            }
            $data = roborock_setCleanMotorMode(
                (int)$cfg['fan_power'],
                (int)$cfg['mop_mode'],
                (int)$cfg['water_box_mode']
            );
            break;

        // Режимы мойки пола
        case 'get_mop_mode':
            $data = roborock_getMopMode();
            break;
        case 'set_mop_mode':
            if (!isset($_REQUEST['mode'])) {
                throw new \Exception('Не передан параметр mode.');
            }
            $data = roborock_setMopMode((int)$_REQUEST['mode']);
            break;

        // Кастомная подача воды
        case 'get_water_box_custom_mode':
            $data = roborock_getWaterBoxCustomMode();
            break;
        case 'set_water_box_custom_mode':
            if (!isset($_REQUEST['param'])) {
                throw new \Exception('Не передан параметр param.');
            }
            $assoc = json_decode($_REQUEST['param'], true);
            if (!is_array($assoc) || !isset($assoc['water_box_mode'])) {
                throw new \Exception('Неверный формат param для set_water_box_custom_mode.');
            }
            $data = roborock_setWaterBoxCustomMode((int)$assoc['water_box_mode']);
            break;

        // Детская блокировка
        case 'get_child_lock_status':
            $data = roborock_getChildLockStatus();
            break;
        case 'set_child_lock_status':
            if (!isset($_REQUEST['param'])) {
                throw new \Exception('Не передан параметр param.');
            }
            $assoc = json_decode($_REQUEST['param'], true);
            if (!is_array($assoc) || !isset($assoc['lock_status'])) {
                throw new \Exception('Неверный формат param для set_child_lock_status.');
            }
            // преобразуем 1/0 в bool
            $data = roborock_setChildLockStatus((bool)$assoc['lock_status']);
            break;

        // Чистка ковра
        case 'get_carpet_clean_mode':
            $data = roborock_getCarpetCleanMode();
            break;
        case 'set_carpet_clean_mode':
            if (!isset($_REQUEST['param'])) {
                throw new \Exception('Не передан параметр param.');
            }
            $assoc = json_decode($_REQUEST['param'], true);
            if (!is_array($assoc) || !isset($assoc['carpet_clean_mode'])) {
                throw new \Exception('Неверный формат param для set_carpet_clean_mode.');
            }
            $data = roborock_setCarpetCleanMode((bool)$assoc['carpet_clean_mode']);
            break;

        // Сбор пыли
        case 'get_dust_collection_mode':
            $data = roborock_getDustCollectionMode();
            break;
        case 'set_dust_collection_mode':
            if (!isset($_REQUEST['mode'])) {
                throw new \Exception('Не передан параметр mode.');
            }
            $data = roborock_setDustCollectionMode((bool)$_REQUEST['mode']);
            break;

        // Основные команды уборки
        case 'app_start':
            $data = roborock_appStart();
            break;
        case 'app_pause':
            $data = roborock_appPause();
            break;
        case 'app_stop':
            $data = roborock_appStop();
            break;
        case 'app_charge':
            $data = roborock_appCharge();
            break;
        case 'app_clean_rooms':
            $rooms = $_POST['rooms'] ?? [];
            if (!is_array($rooms) || count($rooms) === 0) {
                throw new \Exception('Не переданы ID комнат для уборки.');
            }
            $rooms = array_map('intval', $rooms);
            $data  = roborock_appCleanRooms($rooms);
            break;

        // Сбор пыли и мойка
        case 'app_start_collect_dust':
            $data = roborock_appStartCollectDust();
            break;
        case 'app_stop_collect_dust':
            $data = roborock_appStopCollectDust();
            break;
        case 'app_start_wash':
            $data = roborock_appStartWash();
            break;
        case 'app_stop_wash':
            $data = roborock_appStopWash();
            break;

        // Найти меня
        case 'find_me':
            $data = roborock_findMe();
            break;

        default:
            // любой другой raw-метод
            $data = roborock_sendRaw($action);
            break;
    }

    $response['success'] = true;
    $response['data']    = $data;

} catch (\Throwable $e) {
    $response['error'] = $e->getMessage();
}

echo json_encode($response, JSON_UNESCAPED_UNICODE);
exit;
